#version 330

in vec2 TexCoords;
in vec3 fragPos;
in vec4 LightSpacePos;

in vec4 worldPosition;  
in vec3 worldNormal;   

out vec4 color;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;
uniform sampler2DShadow gShadowMap;    
uniform vec4 stone_color;
uniform vec3 mat_ambient;
uniform vec3 mat_diffuse;
uniform vec3 mat_specular;    
uniform float mat_shininess;
uniform vec3 light_position;
uniform vec3 light_ambient;
uniform vec3 light_diffuse;
uniform vec3 light_specular;
uniform vec3 viewPos;

#define EPSILON 0.00009

float CalcShadowFactor(vec4 LightSpacePos)
{
    vec3 ProjCoords = LightSpacePos.xyz / LightSpacePos.w;
    vec2 UVCoords;
    UVCoords.x = 0.5 * ProjCoords.x + 0.5;
    UVCoords.y = 0.5 * ProjCoords.y + 0.5;
    float z = 0.5 * ProjCoords.z + 0.5;

    float xOffset = 1.0/800;
    float yOffset = 1.0/800;

    float Factor = 0.0;
	//gShadowMap= gShadowMap-0.01f;
    for (int y = -1 ; y <= 1 ; y++) {
        for (int x = -1 ; x <= 1 ; x++) {
            vec2 Offsets = vec2(x * xOffset, y * yOffset);
            vec3 UVC = vec3(UVCoords + Offsets, z - EPSILON);
            Factor += texture(gShadowMap, UVC);
        }
    }

    return (0.5 + (Factor / 18.0));


} 


void main()
{
    vec3 ambient = light_ambient * mat_ambient;
    vec3 norm = normalize(worldNormal);
	vec3 position = worldPosition.xyz - worldPosition.w;
    vec3 lightDir = normalize(light_position - fragPos);
    float diff = max(dot(norm, lightDir), 0.0f);
    vec3 diffuse = light_diffuse * (diff * mat_diffuse);
    
    vec3 viewDir = normalize(viewPos - fragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0f), mat_shininess);
    vec3 specular = light_specular * (spec * mat_specular);  
        
    vec3 result = ambient + diffuse + specular;

	float shadowFactor=CalcShadowFactor(LightSpacePos);


	if(vec4(texture(texture_diffuse1, TexCoords)).x>0.0){
	color = vec4(shadowFactor*result, 1.0f)* (vec4(texture(texture_diffuse1, TexCoords)));
	}else{
    color = (shadowFactor)* vec4(result, 1.0f)* (vec4(texture(texture_diffuse1, TexCoords)));
	}

	
}


