#version 330 core

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal; 
layout (location = 2) in vec2 texCoords;
layout (location = 3) in ivec4 boneIndices; 
layout (location = 4) in vec4 boneWeight;

uniform mat4 projection;
uniform mat4 model;
uniform mat4 view;
uniform mat4 bones[60];

uniform mat4 light_model;
uniform mat4 light_view;
uniform mat4 light_projection;

out vec4 LightSpacePos;
out vec2 TexCoords;

out vec3 fragPos;
out vec4 worldPosition;
out vec3 worldNormal;

void main()
{ 
	vec4 newPosition;
	
	if( (boneWeight.x>0 || boneWeight.y>0 || boneWeight.z>0 || boneWeight.w>0) &&
	     (boneIndices.x>0 || boneIndices.y>0 || boneIndices.z>0 || boneIndices.w>0) ){

		vec4 boneWeights = boneWeight;
		boneWeights.w = 1.0 - dot(boneWeights.xyz, vec3(1.0,1.0,1.0));

		mat4 transformMatrix = boneWeights.x * bones[int(boneIndices.x)] ;
		transformMatrix +=  boneWeights.y * bones[int(boneIndices.y)] ;
		transformMatrix +=  boneWeights.z * bones[int(boneIndices.z)] ;
		transformMatrix +=  boneWeights.w * bones[int(boneIndices.w)] ;

		newPosition = transformMatrix * vec4(position, 1.0f);
		gl_Position = projection * view * model * newPosition;

		vec4 newNormal = transformMatrix * vec4(normal, 0.0);
		worldNormal = (model * newNormal).xyz;

	}
	else {
		gl_Position = projection * view * model *vec4(position, 1.0f);
		worldNormal = normal;
	}
	
	worldPosition = model * newPosition;

	LightSpacePos = light_projection * light_view * model * vec4(position, 1.0f); 
	TexCoords = texCoords;
	fragPos = vec3(model * vec4(position, 1.0f));
}